---------------------------------------------------------------------------------------
-- Nom fichier      : Agenda.adb
-- Auteur           : Sylvaine MASSON & Amaury LANGUILLAT
--                    (groupes E et D)
-- Date             : 17/12/2008
--
-- But              :  une application pour une mini gestion d'un agenda electronique
--
-- Date de modif.   :
-- Raison           :
-- Modules appeles  : Ada.Integer_Text_IO, Ada.Strings.Fixed
--
-- Mat. particulier :
-- Compilation      :  Gnat
-- Edition de liens :  Gnat
-- Mode d'execution :  Console
----------------------------------------------------------------------------------------

with Ada.Integer_Text_Io;
use Ada.Integer_Text_Io;
with Ada.Strings.Fixed;
use Ada.Strings.Fixed;

package body Agenda is

   --***********************************************--

   ----------------------------------------------------------------------------
   -- Nom         : Ajouter
   -- But         :
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Ajouter (
         Carnet : in out T_Carnet) is
      Personne : T_Personne;
      Reponse  : Boolean;
   begin
      Saisir(Personne, Carnet, Reponse);
      if Reponse = True then
         if (Carnet.Nb_Personnes < Carnet.Tab'Length) then      -- Contrle de la place restante dans le carnet
            Carnet.Nb_Personnes := Carnet.Nb_Personnes+1;
            Carnet.Tab(Carnet.Nb_Personnes) := Personne;
            Ordonner(Carnet);
         else
            raise Taille_Max_Error;
         end if;
      end if;
   end Ajouter;

   ----------------------------------------------------------------------------
   -- Nom         : Supprimer
   -- But         :
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Supprimer (
         Carnet : in out T_Carnet) is
      Numpersonne : Positive;
      Personne    : T_Personne;

   begin
      if  Carnet.Nb_Personnes = 0 then
         Put_Line(" Agenda vide ou non ouvert ");
         Put(" Appuyez sur Entree pour continuer ");
         Skip_Line;
      else

         Put_Line(
            " Entrez le nom et le prenom de la personne a supprimer ");
         Saisir(" Nom       : ", Personne.Nom);
         Saisir(" Prenom    : ", Personne.Prenom);
         if Est_Inclue(Personne, Carnet) then                  -- Recherche du contact dans le carnet
            Numpersonne := Indice(Personne,Carnet);
            Carnet.Nb_Personnes := Carnet.Nb_Personnes-1;
            Carnet.Tab(Numpersonne..Carnet.Nb_Personnes) := Carnet.Tab(
               Numpersonne+1..Carnet.Nb_Personnes+1);
         else
            Put_Line(
               " Cette personne n'est pas repertoriee dans ce carnet !");
            Put(" Appuyez sur Entree pour continuer ");
            Skip_Line;
         end if;
      end if;
   end Supprimer;

   ----------------------------------------------------------------------------
   -- Nom         : Modifier
   -- But         :
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Modifier (
         Carnet : in out T_Carnet) is
      Personne : T_Personne;
      Char     : Character;
   begin
      if  Carnet.Nb_Personnes = 0 then                  -- Verification qu'un carnet est deja ouvert
         Put_Line(" Vous n'avez ouvert aucun agenda ");
         Put_Line("Retournez au menu, et choisissez le choix 1 pour le faire ");
         Put(" Appuyez sur Entree pour continuer ");
         Get_Immediate(Char);
      else
         Put_Line(
            " Entrez le nom et le prenom de la personne a modifier ");
         Saisir(" Nom       : ", Personne.Nom);
         Saisir(" Prenom    : ", Personne.Prenom);
         if Est_Inclue(Personne, Carnet) then
            Modifier(Personne , Carnet);
         else
            Put_Line(
               " Cette personne n'est pas repertoriee dans ce carnet ! ");
            Put(" Appuyez sur Entree pour continuer ");
            Skip_Line;
         end if;
      end if;
   end Modifier;

   ----------------------------------------------------------------------------
   -- Nom         : Afficher
   -- But         : Afficher le carnet
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Afficher (
         Carnet : in     T_Carnet) is
   begin
      Put_Line("------- Contenu de votre Agenda -------");
      Put(Carnet.Nb_Personnes);
      Put(" entree(s) dans votre Agenda.");
      New_Line;
      for I in 1..Carnet.Nb_Personnes loop
         Put_Line("***********************");
         Afficher(Carnet.Tab(I));
         New_Line;
      end loop;
   end Afficher;

   ----------------------------------------------------------------------------
   -- Nom         : Enregistrer
   -- But         :
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Enregistrer (
         Carnet      : in     T_Carnet;
         Nom_Fichier : in out T_Nom_Long;
         Leng_Path   : in out Natural) is
      Fichier_Texte : File_Type;
   begin

      if Leng_Path = 0  then            
         Put(" Entrez le nom de votre fichier : ");
         Get_Line(Nom_Fichier, Leng_Path);
         Insert(Nom_Fichier,Leng_Path+1,".agd");
         Create(Fichier_Texte, Out_File, Nom_Fichier);
      else
         Open(Fichier_Texte, Out_File, Nom_Fichier);
      end if;

      Put(Fichier_Texte,"Ce carnet contient ");
      Put(Fichier_Texte,Carnet.Nb_Personnes,0);
      Put(Fichier_Texte," Personne(s) ");
      New_Line(Fichier_Texte);

      for I in 1..Carnet.Nb_Personnes loop
         Put_Line(Fichier_Texte,"------------------------------------");
         Enregistrer(Fichier_Texte,Carnet.Tab(I));
         New_Line(Fichier_Texte);
      end loop;
      Close(Fichier_Texte);

   exception
      when Name_Error =>
         Put(" Le fichier n'existe pas ");
         Leng_Path := 0;
         Nom_Fichier := (others=>' ');
         New_Line;

      when others =>
         Close(Fichier_Texte);
         Leng_Path := 0;
         Nom_Fichier := (others=>' ');

         Put_Line(" Erreur de sauvegarde ");
   end Enregistrer;

   ----------------------------------------------------------------------------
   -- Nom         : Ouvrir
   -- But         :
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Ouvrir (
         Carnet      :    out T_Carnet;
         Nom_Fichier : in out T_Nom_Long;
         Leng_Path   : in out Natural) is
      Fichier_Texte : File_Type;
      Position      : Natural;
      Str           : String (1 .. 80);
      Blanc         : Character;
   begin

      if Leng_Path /= 0  then
         Put(
            " Un ficher est deja ouvert. Retournez dans le menu principal pour le fermer ");
      else
         Put(" Entrez le nom de votre fichier : ");
         Get_Line(Nom_Fichier, Leng_Path);
         Insert(Nom_Fichier,Leng_Path+1,".agd");
         Open(
            File => Fichier_Texte,
            Mode => In_File,
            Name => Nom_Fichier);

         Skip_Line(Fichier_Texte);
         Get(Fichier_Texte, Carnet.Nb_Personnes);
         Skip_Line(Fichier_Texte);

         for I in 1..Carnet.Nb_Personnes loop
            Get_Line(Fichier_Texte, Str, Position);
            Get(Fichier_Texte, Carnet.Tab(I).Nom);
            Get(Fichier_Texte, Carnet.Tab(I).Prenom);
            Get(Fichier_Texte,Carnet.Tab(I).Sexe);
            Skip_Line(Fichier_Texte);
            Get(Fichier_Texte,Carnet.Tab(I).Telephone);
            Skip_Line(Fichier_Texte);
            Get(Fichier_Texte,Carnet.Tab(I).Email);
            Skip_Line(Fichier_Texte);
            Get(Fichier_Texte, Carnet.Tab(I).Adresse.Numero);
            Get(Fichier_Texte, Blanc);
            Get(Fichier_Texte, Carnet.Tab(I).Adresse.Nom_Rue);
            Skip_Line(Fichier_Texte);
            Get(Fichier_Texte, Carnet.Tab(I).Adresse.Code_Postal);
            Get(Fichier_Texte, Blanc);
            Get_Line(Fichier_Texte, Carnet.Tab(I).Adresse.Nom_Ville,
               Position);
            Skip_Line(Fichier_Texte);
         end loop;
         Close(Fichier_Texte);
      end if;

   exception
      when Data_Error|Constraint_Error =>
         New_Line;
         Put_Line(" Erreur de lecture dans "& Nom_Fichier);
         Close(Fichier_Texte);
         Carnet.Nb_Personnes := 0;
         Leng_Path := 0;
         Nom_Fichier := (others=>' ');

         raise File_Read_Error;

      when others =>
         New_Line;
         Put_Line(Nom_Fichier(1..Leng_Path+4) & " n'existe pas ");
         Carnet.Nb_Personnes := 0;
         Leng_Path := 0;
         Nom_Fichier := (others=>' ');

         raise File_Open_Error;
   end Ouvrir;

   ----------------------------------------------------------------------------
   -- Nom         : Fermer
   -- But         :
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Fermer (
         Carnet      : in out T_Carnet;
         Nom_Fichier : in out T_Nom_Long;
         Leng_Path   : in out Natural) is
   begin

      if Carnet.Nb_Personnes = 0 and Leng_Path = 0 then
         Put_Line(" Aucun agenda n'est ouvert ! ");
      else
         Quitter(Carnet, Nom_Fichier, Leng_Path);
         Carnet.Nb_Personnes := 0;
         Leng_Path := 0;
         Nom_Fichier := (others=>' ');
      end if;
   end Fermer;

   ----------------------------------------------------------------------------
   -- Nom         : Quitter
   -- But         : Quitter le programme mais demande si l'utilisateur veut sauvegarder ses modifications
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Quitter (
         Carnet      : in out T_Carnet;
         Nom_Fichier : in out T_Nom_Long;
         Leng_Path   : in out Natural) is
   begin
      if Carnet.Nb_Personnes /= 0 or Leng_Path /= 0 then
         if Carnet.Nb_Personnes /= 0 then
            if Saisir(
                  " Vous avez saisi un nouvel agenda, voulez-vous l'enregistrer ? [O/N] ") =
                  O then
               Enregistrer(Carnet, Nom_Fichier, Leng_Path);
            end if;
         else
            if Saisir(
                  " Vous avez ouvert un agenda, voulez-vous l'enregistrer ? [O/N] ") =
                  O then
               Enregistrer(Carnet, Nom_Fichier, Leng_Path);
            end if;
         end if;
      end if;
   end Quitter;

   ----------------------------------------------------------------------------
   -- Nom         : Est_Inclue
   -- But         : Verifier si une personne existe deja dans l'agenda
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   function Est_Inclue (
         Personne : in     T_Personne;
         Carnet   : in     T_Carnet)
     return Boolean is
      Repo : Boolean;
   begin
      Repo := False;
      for I in 1..Carnet.Nb_Personnes loop
         if Personne.Nom = Carnet.Tab(I).Nom then
            if Personne.Prenom = Carnet.Tab(I).Prenom then
               Repo := True;
               exit;
            end if;
         end if;
         exit when Personne.Nom < Carnet.Tab(I).Nom;
      end loop;
      return Repo;
   end Est_Inclue;

   ----------------------------------------------------------------------------
   -- Nom         : Saisir
   -- But         : Demande d'une reponse [O/N]
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   function Saisir (
         Str : in     String)
     return T_Reponse is
      K : T_Reponse;
   begin
      loop
         begin
            Put(Str);
            Get(K);
            Skip_Line;
            return K;
         exception
            when Data_Error =>
               Put_Line(" Mauvaise entree, recommencez svp !");
         end;
      end loop;
   end Saisir;

   ----------------------------------------------------------------------------
   -- Nom         : Indice
   -- But         : Retourne l'indice d'une personne donnee en parametre
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   function Indice (
         Personne : in     T_Personne;
         Carnet   : in     T_Carnet)
     return Natural is
      Ind : Natural;
   begin
      for I in 1..Carnet.Nb_Personnes loop
         if Personne.Nom = Carnet.Tab(I).Nom and Personne.Prenom =
               Carnet.Tab(I).Prenom then
            Ind := I;
         end if;
         exit when Personne.Nom < Carnet.Tab(I).Nom;
      end loop;
      return Ind;
   end Indice;

   ----------------------------------------------------------------------------
   -- Nom         : Saisir
   -- But         : Saisir un nombre inferieur a une valeur passee en parammetre
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Saisir (
         Str     : in     String;
         Val_Max :        Natural;
         Val     :    out Positive) is
   begin
      loop
         begin
            Put(Str);
            Get(Val);
            Skip_Line;
            if Val >= 1 and Val <= Val_Max then
               exit;
            else
               raise Taille_Max_Error;
            end if;

         exception
            when Taille_Max_Error =>
               Put_Line("Entrez un nombre entre 1 et " & Natural'Image(
                     Val_Max) & " Svp");
            when Constraint_Error =>
               Put_Line(" Mauvaise entree, recommencez svp !");
         end;
      end loop;
   end Saisir;

   ----------------------------------------------------------------------------
   -- Nom         : Saisir
   -- But         : Demande d'une variable donnee en parametre
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Saisir (
         Str      : in     String;
         Variable :    out String) is
      Entier : Integer;
   begin
      Put(Str);
      Get_Line(Variable, Entier);
   end Saisir;

   ----------------------------------------------------------------------------
   -- Nom         : Saisir
   -- But         : Demande des donnees d'une personne
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Saisir (
         Personne :    out T_Personne;
         Carnet   : in out T_Carnet;
         Reponse  : in out Boolean) is
   begin
      Put_Line("Entrez les donnees de la personne :");
      Saisir("Nom : ", Personne.Nom);
      Saisir("Prenom : ", Personne.Prenom);
      if not Est_Inclue(Personne, Carnet) then
         Saisir("Sexe : ", Personne.Sexe);
         Saisir("Telephone : ", Personne.Telephone);
         Saisir("Email : ", Personne.Email);
         Saisir("Adresse : ", Personne.Adresse);
         Reponse := True; -- Validation de l'entre de la personne
      else
         if Saisir(
               "La personne existe deja, voulez-vous la modifier? [O/N]") =
               O then
            Modifier(Personne, Carnet);
         end if;
      end if;
   end Saisir;

   ----------------------------------------------------------------------------
   -- Nom         : Saisir
   -- But         : Demande du sexe d'une personne
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Saisir (
         Str  : in     String;
         Sexe :    out T_Sexe) is
   begin
      loop
         begin
            Put(Str);
            Get(Sexe);
            Skip_Line;
            exit;
         exception
            when Data_Error =>
               Put_Line("Tapez M ou F pour selectionner le sexe");
         end;
      end loop;
   end Saisir;

   ----------------------------------------------------------------------------
   -- Nom         : Saisir
   -- But         : Demande l'adresse d'une personne
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Saisir (
         Str     : in     String;
         Adresse : in out T_Adresse) is
   begin
      Put_Line(Str);
      Saisir("Entrez le numero :", 1000, Adresse.Numero);
      Saisir("Nom de la rue :", Adresse.Nom_Rue);
      Saisir("Code Postal :", 99999, Adresse.Code_Postal);
      Saisir("Nom de la ville :", Adresse.Nom_Ville);
   end Saisir;

   ----------------------------------------------------------------------------
   -- Nom         : Ordonner
   -- But         : Ordonner les contacts dans l'ordre alphabetique
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Ordonner (
         Carnet : in out T_Carnet) is
      procedure Permuter (
            V1,
            V2 : in out T_Personne) is
         Temp : T_Personne;
      begin
         Temp := V1;
         V1   := V2;
         V2   := Temp;
      end Permuter;

   begin
      if (Carnet.Nb_Personnes > 1) then
         for M in 1.. Carnet.Nb_Personnes-1 loop
            for J in 1.. Carnet.Nb_Personnes-M loop
               if Carnet.Tab(J).Nom>Carnet.Tab(J+1).Nom then
                  Permuter(Carnet.Tab(J),Carnet.Tab(J+1));
               elsif Carnet.Tab(J).Nom = Carnet.Tab(J+1).Nom then
                  if Carnet.Tab(J).Prenom>Carnet.Tab(J+1).Prenom then
                     Permuter(Carnet.Tab(J),Carnet.Tab(J+1));
                  end if;
               end if;
            end loop;
         end loop;
      end if;
   end Ordonner;

   ----------------------------------------------------------------------------
   -- Nom         : Modifier
   -- But         : Modifier les donnees d'un contact
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Modifier (
         Personne : in out T_Personne;
         Carnet   : in out T_Carnet) is
      type T_Menu1 is
            (Adresse,
             Email,
             Telephone,
             Quitter);
      package Menu1_Io is new Enumeration_Io(T_Menu1);
      use Menu1_Io;
      subtype T_Choix is Positive range 1..4;
      Choice : T_Choix;
      Ind    : Positive;
   begin
      loop
         Put_Line(
            "  ********** Quel parametre voulez-vous modifier ? ***********");
         Put_Line(
            "  *                                                          *");
         for Menu in T_Menu1 loop
            Put("  *         ");
            Put(Menu, T_Menu1'Width);
            Put(" -------------------------:  ");
            Put(T_Menu1'Pos(Menu)+1, 0);
            Put("          *");
            New_Line;
         end loop;
         Put_Line(
            "  *                                                          * ");
         Put_Line(
            "  ************************************************************ ");
         New_Line(3);
         Put("Saisissez votre choix : ");
         loop
            begin
               Get(Choice);
               New_Line(2);
               Skip_Line;
            exception
               when Data_Error =>
                  Put_Line("Erreur du choix dans le menu ! Rentrez un nouveau choix");
               end;
         end loop;

         Ind := Indice(Personne, Carnet);

         case Choice is
            when 1 =>
               Saisir("Entrez la nouvelle adresse", Carnet.Tab(Ind).
                  Adresse);
            when 2 =>
               Saisir("Saisissez le nouvel email : ", Carnet.Tab(
                     Ind).Email);
            when 3 =>
               Saisir("Saisissez le nouveau numero de telephone : ",
                  Carnet.Tab(Ind).Telephone);
            when 4 =>
               exit;
         end case;
      end loop;
   end Modifier;

   ----------------------------------------------------------------------------
   -- Nom         : Enregistrer
   -- But         : Enregistrer les donnees d'un contact
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Enregistrer (
         Fichier_Texte :        File_Type;
         Personne      : in     T_Personne) is
   begin
      Put(Fichier_Texte, Personne.Nom);
      Put(Fichier_Texte, Personne.Prenom);
      Put(Fichier_Texte, Personne.Sexe);
      Put(Fichier_Texte, Personne.Telephone);
      Put(Fichier_Texte, Personne.Email);
      Enregistrer(Fichier_Texte, Personne.Adresse);
   end Enregistrer;

   ----------------------------------------------------------------------------
   -- Nom         : Enregistrer
   -- But         : Enregistrer l'adresse d'un contact
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Enregistrer (
         Fichier_Texte :        File_Type;
         Adresse       : in     T_Adresse) is
   begin
      Put(Fichier_Texte, Adresse.Numero);
      Put(Fichier_Texte, ", ");
      Put(Fichier_Texte, Adresse.Nom_Rue);
      Put(Fichier_Texte, ", ");
      Put(Fichier_Texte, Adresse.Code_Postal);
      Put(Fichier_Texte, " ");
      Put(Fichier_Texte, Adresse.Nom_Ville);
   end Enregistrer;

   ----------------------------------------------------------------------------
   -- Nom         : Afficher
   -- But         : Afficher l'adresse d'un contact
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Afficher (
         Adresse : in     T_Adresse) is
   begin
      Put("Adresse : ");
      Put(Adresse.Numero);
      Put(", ");
      Put(Adresse.Nom_Rue);
      Put(", ");
      Put(Adresse.Code_Postal);
      Put(" ");
      Put(Adresse.Nom_Ville);
   end Afficher;

   ----------------------------------------------------------------------------
   -- Nom         : Afficher
   -- But         : Afficher les donnees d'un contact
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   procedure Afficher (
         Personne : in     T_Personne) is
   begin
      Put("Nom : ");
      Put(Personne.Nom);
      New_Line;
      Put("Prenom : ");
      Put(Personne.Prenom);
      New_Line;
      Put("Sexe : ");
      Put(Personne.Sexe);
      New_Line;
      Put("Telephone : ");
      Put(Personne.Telephone);
      New_Line;
      Put("Email : ");
      Put(Personne.Email);
      New_Line;
      Afficher(Personne.Adresse);
   end Afficher;

   ------------------------------------------------------------------------

end Agenda;