---------------------------------------------------------------------------------------
-- Nom fichier      : Agenda.ads
-- Auteur           : Sylvaine MASSON & Amaury LANGUILLAT
--                    (groupes E et D)
-- Date             : 17/12/2008
--
-- But              :  Paquetage de specification pour une mini gestion d'un agenda electronique
--
-- Date de modif.   :
-- Raison           :
-- Modules appeles  : Ada.Text_IO
--
-- Mat. particulier :
-- Compilation      :  Gnat
-- Edition de liens :  Gnat
-- Mode d'execution :  Console
----------------------------------------------------------------------------------------

with Ada.Text_Io;
use Ada.Text_Io;

package Agenda is

   Taille_Max_Error : exception;
   File_Open_Error  : exception;
   File_Read_Error  : exception;
   File_Write_Error : exception;

   Max_Taille       : constant Positive := 50;
   Taille_Nom_Court : constant Positive := 25;
   Taille_Nom_Long  : constant Positive := 50;

   type T_Reponse is
         (O,
          N);
   package Reponse_Io is new Enumeration_Io(T_Reponse);
   use Reponse_Io;

   type T_Sexe is
         (M,
          F);
   package Sexe_Io is new Enumeration_Io(T_Sexe);
   use Sexe_Io;

   subtype T_Nom_Court is String(1..Taille_Nom_Court);
   subtype T_Nom_Long  is String(1..Taille_Nom_Long);

   type T_Adresse is
      record
         Numero      : Positive;
         Nom_Rue     : T_Nom_Long := (others => ' ');
         Code_Postal : Positive;
         Nom_Ville   : T_Nom_Long := (others => ' ');
      end record;

   type T_Personne is
      record
         Nom       : T_Nom_Court := (others => ' ');
         Prenom    : T_Nom_Court := (others => ' ');
         Sexe      : T_Sexe      := M;
         Telephone : T_Nom_Long  := (others => ' ');
         Email     : T_Nom_Long  := (others => ' ');
         Adresse   : T_Adresse;
      end record;

   type T_Table is array (Positive range <>) of T_Personne;

   type T_Carnet
         (Long : Positive) is
      record
         Tab          : T_Table (1 .. Long);
         Nb_Personnes : Natural             := 0;
         Nom_Fichier  : T_Nom_Long          := (others => ' '); -- variable pour le nom du fichier physique
         Leng_Path    : Natural             := 0;               -- variable pour la longueur du fichier physique
      end record;

   Nom_Fichier : T_Nom_Long := (others => ' '); -- variable pour le nom du fichier physique
   Leng_Path   : Natural    := 0;               -- variable pour la longueur du fichier physique


   --***************************************************************************
   -- Specifications des procedures et fonctions utilises
   --***************************************************************************
   -- Ajouter une personne a l'agenda
   procedure Ajouter (
         Carnet : in out T_Carnet);

   -- Supprimer une personne dont le nom est a saisir par le clavier
   procedure Supprimer (
         Carnet : in out T_Carnet);

   -- Modifier une personne dont le nom est a saisir par le clavier
   procedure Modifier (
         Carnet : in out T_Carnet);

   -- Afficher un contact de l'agenda
   procedure Afficher (
         Carnet : in     T_Carnet);

   -- Enregistrer l'agenda
   procedure Enregistrer (
         Carnet      : in     T_Carnet;
         Nom_Fichier : in out T_Nom_Long;
         Leng_Path   : in out Natural);

   -- Ouvrir un agenda existant
   procedure Ouvrir (
         Carnet      :    out T_Carnet;
         Nom_Fichier : in out T_Nom_Long;
         Leng_Path   : in out Natural);

   -- Fermer l'agenda ouvert
   procedure Fermer (
         Carnet      : in out T_Carnet;
         Nom_Fichier : in out T_Nom_Long;
         Leng_Path   : in out Natural);

   -- Quitter le programme
   procedure Quitter (
         Carnet      : in out T_Carnet;
         Nom_Fichier : in out T_Nom_Long;
         Leng_Path   : in out Natural);


private
   -- Methodes non visibiles de l'exterieur
   -- elle sont appelees uniquement par les methodes de ce paquetage

   -- voir si une personne est dans le carnet
   function Est_Inclue (
         Personne : in     T_Personne;
         Carnet   : in     T_Carnet)
     return Boolean;

   -- Saisir_Reponse appele par modifier reponse pour une confirmation par O/N
   function Saisir (
         Str : in     String)
     return T_Reponse;

   -- retourner l'index d'une personne dans le carnet
   function Indice (
         Personne : in     T_Personne;
         Carnet   : in     T_Carnet)
     return Natural;

   -- Saisir un nombre inferieur a une valeur passee en parammetre
   procedure Saisir (
         Str     : in     String;
         Val_Max :        Natural;
         Val     :    out Positive);

   -- Saisir une chaine de caractere
   procedure Saisir (
         Str      : in     String;
         Variable :    out String);

   -- Saisir une personne
   procedure Saisir (
         Personne :    out T_Personne;
         Carnet   : in out T_Carnet;
         Reponse  : in out Boolean);

   -- Saisir le sexe de la personne
   procedure Saisir (
         Str  : in     String;
         Sexe :    out T_Sexe);

   -- Saisir une adresse
   procedure Saisir (
         Str     : in     String;
         Adresse : in out T_Adresse);

   -- Ordonner le carnet apres chaque ajout d'une personne dans le carnet
   procedure Ordonner (
         Carnet : in out T_Carnet);

   -- Modifier une personne dans le carnet
   procedure Modifier (
         Personne : in out T_Personne;
         Carnet   : in out T_Carnet);

   -- Enregistrer une personne
   procedure Enregistrer (
         Fichier_Texte :        File_Type;
         Personne      : in     T_Personne);

   -- Enregistrer une adresse
   procedure Enregistrer (
         Fichier_Texte :        File_Type;
         Adresse       : in     T_Adresse);

   -- Afficher une personne
   procedure Afficher (
         Personne : in     T_Personne);

   -- Afficher une adresse
   procedure Afficher (
         Adresse : in     T_Adresse);

end Agenda;