---------------------------------------------------------------------------------------
-- Nom fichier      : Agenda_Manager.adb
-- Auteur           : Sylvaine MASSON & Amaury LANGUILLAT
--                    (groupes E et D)
-- Date             : 17/12/2008
--
-- But              :  une application pour une mini gestion d'un agenda electronique
--
-- Date de modif.   :
-- Raison           :
-- Modules appeles  : Agenda paquetage des operations sur l'agenda
--
-- Mat. particulier :
-- Compilation      :  Gnat
-- Edition de liens :  Gnat
-- Mode d'execution :  Console
----------------------------------------------------------------------------------------

with Ada.Text_Io;
use Ada.Text_Io;
with Ada.Integer_Text_Io;
use Ada.Integer_Text_Io;
with Agenda;
use Agenda;

procedure Agenda_Manager is

   type T_Menu is
         (Ouvrir,
          Ajouter,
          Supprimer,
          Modifier,
          Afficher,
          Enregistrer,
          Fermer,
          Quitter);
   package Menu_Io is new Enumeration_Io(T_Menu);
   use Menu_Io;


   ------------  Specifications ----------------------------------
   procedure Presentation;
   procedure Afficher;
   function Menu return T_Menu;

   -----------------------------------------------------------------------------
   -- Nom         : Presentation
   -- But         :
   -- Parametres  :
   -----------------------------------------------------------------------------
   procedure Presentation is
   begin -- Presentation
      Put_Line("*************************************************************************");
      Put_Line("* Programme qui permet de faire une mini gestion d'un carnet de contact *");
      Put_Line("*                                                                       *");
      Put_Line("*************************************************************************");
   end Presentation;

   ----------------------------------------------------------------------------
   -- Nom         : Afficher
   -- But         : Afficher le menu ci-dessous
   -- 	New_Line;
   -- 	Put_Line(" ***** Quelle operation desirez-vous effectuer ? *******");
   -- 	Put_Line(" *                                                     *");
   -- 	Put_Line(" *   Ajouter--------------------------------------: 1  *");
   -- 	Put_Line(" *   Supprimer------------------------------------: 2  *");
   --       Put_Line(" *   Modifier-------------------------------------: 3  *");
   -- 	Put_Line(" *   Afficher-------------------------------------: 4  *");
   -- 	Put_Line(" *   Enregistrer----------------------------------: 5  *");
   -- 	Put_Line(" *   Ouvrir---------------------------------------: 6  *");
   --       Put_Line(" *   Fermer---------------------------------------: 7  *");
   -- 	Put_Line(" *   Quitter--------------------------------------: 8  *");
   -- 	Put_Line(" *                                                     *");
   -- 	Put_Line(" *******************************************************");
   --       New_line;

   -----------------------------------------------------------------------------
   procedure Afficher is
   begin
      New_Line;
      Put_Line(" ******* Quelle operation desirez-vous effectuer ? ******* ");
      New_Line;
      for Menu in T_Menu loop
         Put("     ");
         Put(Menu, T_Menu'Width);
         Put(" -------------------------------  ");
         Put(T_Menu'Pos(Menu)+1, 0);
         New_Line;
      end loop;
      New_Line;
      Put_Line(" ********************************************************* ");
      New_Line;
   end Afficher;

   ----------------------------------------------------------------------------
   -- Nom         : Menu
   -- But         :
   -- Parametres  :
   -- Exception   :
   -----------------------------------------------------------------------------
   function Menu return T_Menu is
      subtype T_Choix is Positive range T_Menu'Pos(T_Menu'First)+1..T_Menu'Pos(T_Menu'Last)+1;
      Choix : T_Choix;
   begin -- Menu
      Afficher;
      Put(" Saisissez votre Choix : ");
      loop
         begin
            Get(Choix);
            New_Line(2);
            Skip_Line;
            return T_Menu'Val(Choix-1);
         exception
            when Data_Error | Constraint_Error=>
               Skip_Line;
               New_Line;
               Put_Line(" Saisie incorrecte ");
               Put(" Veuillez entrer une nouvelle valeur : ");
         end;
      end loop;
   end Menu;
   --------------------------------------------------------------------------------------------

   -- Variables
   Choix  : T_Menu;                -- choix de l'utilisateur
   Carnet : T_Carnet (Max_Taille);

   ------------------------------------------------------------------------------
   -- PROGRAMME PRINCIPAL
   ------------------------------------------------------------------------------
begin -- Agenda_Manager
   Presentation;		-- appel de la fonction presentation
   loop
      begin
         Choix:=Menu;  	-- appel de la fonction Menu

         -- Execute selon le choix
         case Choix is
            when T_Menu'(Ajouter)     =>
               Ajouter(Carnet);
            when T_Menu'(Supprimer)   =>
               Supprimer(Carnet);
            when T_Menu'(Modifier)    =>
               Modifier(Carnet);
            when T_Menu'(Afficher)    =>
               Afficher(Carnet);
            when T_Menu'(Enregistrer) =>
               Enregistrer(Carnet, Nom_Fichier, Leng_Path);
            when T_Menu'(Ouvrir)      =>
               Ouvrir(Carnet, Nom_Fichier,Leng_Path);
            when T_Menu'(Fermer)      =>
               Fermer(Carnet, Nom_Fichier,Leng_Path);
            when T_Menu'(Quitter)     =>
               Quitter(Carnet, Nom_Fichier, Leng_Path);
               exit;
         end case;

      exception
         when Taille_Max_Error=>
            New_Line;
            Put_Line(" Carnet plein ! ");
            Put_Line(" Impossible d'effectuer une nouvelle saisie ");
            Put_Line(" Vous devez reduire le nombre de personnes dans votre agenda ");

         when Constraint_Error=>-- Car si tel est le cas une exception serait generee
            New_Line;
            Put(" Fermeture du programme ");

         when  File_Open_Error=>
            New_Line;
            Put(" Verifiez le nom ou le chemin de votre fichier de donnees ");

         when  File_Read_Error=>
            New_Line;
            Put(" Fichier de donnees incorrect ");

         when  others =>
            New_Line;
            Put(" Erreur inconnue ");

      end;
      New_Line(2);
      Put(" Appuyez sur Entree pour continuer ");
      Skip_Line;
   end loop;
end Agenda_Manager;